PROGRAM compute_fNLcontamination_lensingisw
! A sample code for computing a contamination of the local-form fNL due to
! the lensing-linear Integrated Sachs-Wolfe effect bispectrum or the 
! lensing-non-linear Rees-Sciama effect bispectrum for a full-sky, 
! cosmic-variance only experiment. 
!
! The contamination is computed as
!
! fNL(contamination) = NUM / DENOM
!
! where 
!
! NUM = sum_{2<=l1<=l2<=l3} [B_localform(l1,l2,l3)*B_lensingISW(l1,l2,l3)]
!                             /[C(l1)C(l2)C(l3)Delta(l1,l2,l3)].
!
! DENOM = sum_{2<=l1<=l2<=l3} [B_localform(l1,l2,l3)]^2
!                             /[C(l1)C(l2)C(l3)Delta(l1,l2,l3)].
!
! << CAUTION >>
! Note that the covariance of the bispectrum appearing in the denominator of the 
! above equation ignores contributions from the non-Gaussian signal (i.e., this
! covarianec is valid only when non-Gaussianity is absent). This may affect the 
! value of fNL(contamination) at l>~2200. 
! For details, see Lewis, Challinor & Hanson, JCAP, 03, 018 (2011).
!
! April 9, 2012: E.Komatsu
!
! MODIFIED on April 21, 2012: 
! - More accurate expression for the squeezed-limit bispectrum where the unlensed Cl is
!   replaced by the lensed Cl is implemented following Lewis, Challinor & Hanson, JCAP,
!   03, 018 (2011)  
  IMPLICIT none
  character(len=128) :: filename
  integer :: lmax=3000 ! maximum multipole
  integer :: lmin=2    ! minimum multipole
  integer :: i,j,l,l1,l2,l3,info,dum
  integer :: ir,nr
  double precision :: l1min,l1max,gaunt,fourpi,twopi,blin,bnl,delta
  double precision :: num1a,num1b,num2a,num2b,denom,blocal
  double precision, allocatable, dimension(:) :: cl,clp,wigner3j,qlin,qnl
  double precision, allocatable, dimension(:) :: r,dr
  double precision, allocatable, dimension(:,:) :: alpha,beta
  character(len=70) :: header
  print*,'lmin=',lmin
  twopi=2d0*3.1415926535d0
  fourpi=4d0*3.1415926535d0
  ! read in alpha_l(r), beta_l(r) and C_l
  print*,'filename of alpha and beta?'
  print*,'(calculated from compute_alphabeta: eg, l_r_alpha_beta_lmax=3000.txt)'
  read*,filename
  !!$ filename='l_r_alpha_beta.txt'
  open(1,file=filename,status='old')
  ! skip first 3 rows
  do i=1,3
     read(1,'(1A70)')header
     print*,header
  enddo
  print*,'# of radial points (enter the number shown above: eg, 439)'
  read*,nr
  print*,'maximum multipole (enter the number shown above or less: eg, 3000)'
  read*,lmax
  allocate(r(nr),dr(nr),alpha(lmax,nr),beta(lmax,nr))
! read in the radiation transfer functions
  print*,'read in the radiation transfer functions...'
  do l=2,lmax
     do ir=1,nr
        read(1,*)dum,r(ir),dr(ir),alpha(l,ir),beta(l,ir)
     enddo
  enddo
  close(1)
  print*,'done.'
! read in non-linear Q(l)
  filename='ell_ql_3pt.txt'
  open(1,file=filename,status='old')
  print*,'Q(l) file (non-linear)= ',trim(filename)
  allocate(qnl(lmax))
  do i=2,lmax
     read(1,*)dum,qnl(i)
  enddo
  close(1)
! read in linear Q(l)
  filename='ell_ql_linear.txt'
  print*,'Q(l) file (linear)= ',trim(filename)
  open(1,file=filename,status='old')
  allocate(qlin(lmax))
  do i=2,lmax
     read(1,*)dum,qlin(i)
  enddo
  close(1)
! read in un-lensed, "primordial" Cl
  allocate(clp(lmax))
  filename='wmap5baosn_max_likelihood_scalCls.dat'
  print*,'cl file (unlensed)= ',trim(filename)
  open(2,file=filename,status='old')
  do l=2,lmax
     read(2,*)dum,clp(l)
     clp(l)=clp(l)/dble(l)/(dble(l)+1d0)*twopi/(2.726d6)**2d0
  enddo
  close(2)
! read in lensed Cl
  allocate(cl(lmax))
  filename='wmap5baosn_max_likelihood_lensedCls.dat'
  print*,'cl file (lensed)= ',trim(filename)
  open(2,file=filename,status='old')
  do l=2,lmax
     read(2,*)dum,cl(l)
     cl(l)=cl(l)/dble(l)/(dble(l)+1d0)*twopi/(2.726d6)**2d0
  enddo
  close(2)
! begin computing chi^2
  allocate(wigner3j(2*lmax+1))
  open(3,file='lmax_fNLcont_linear_nonlinear_leading.txt',status='unknown')
  open(4,file='lmax_fNLcont_linear_nonlinear_accurate.txt',status='unknown')
  num1a=0d0
  num2a=0d0
  num1b=0d0
  num2b=0d0
  denom=0d0
  do l3=lmin,lmax
     print*,'l3=',l3,' /',lmax
     do l2=lmin,l3
        call DRC3JJ(dble(l2),dble(l3),0d0,0d0, &
             l1min,l1max,wigner3j,2*lmax+1,info)
        if ((int(l1min).le.l2)) then
           do l1=int(l1min),l2
              if ((l1.ge.2).and.(mod(l1+l2+l3,2).eq.0)) then
                 gaunt=wigner3j(l1-int(l1min)+1)/dsqrt(fourpi) &
                      *dsqrt(dble(2*l1+1))*dsqrt(dble(2*l2+1)) &
                      *dsqrt(dble(2*l3+1))
                 blocal=2d0*gaunt*sum(r(:)*r(:)*dr(:)*( &
                      beta(l1,:)*beta(l2,:)*alpha(l3,:) &
                      +beta(l3,:)*beta(l1,:)*alpha(l2,:) &
                      +beta(l2,:)*beta(l3,:)*alpha(l1,:)))
                 denom=denom+blocal**2d0/cl(l1)/cl(l2)/cl(l3)/delta(l1,l2,l3)
                 ! Leading-order expression
                 blin=0.5d0*gaunt*( &
                      (l1*(l1+1d0)-l2*(l2+1d0)+l3*(l3+1d0))*clp(l1)*qlin(l3) &
                      +(l3*(l3+1d0)-l1*(l1+1d0)+l2*(l2+1d0))*clp(l3)*qlin(l2) &
                      +(l2*(l2+1d0)-l3*(l3+1d0)+l1*(l1+1d0))*clp(l2)*qlin(l1) &
                      +(l1*(l1+1d0)-l3*(l3+1d0)+l2*(l2+1d0))*clp(l1)*qlin(l2) &
                      +(l2*(l2+1d0)-l1*(l1+1d0)+l3*(l3+1d0))*clp(l2)*qlin(l3) &
                      +(l3*(l3+1d0)-l2*(l2+1d0)+l1*(l1+1d0))*clp(l3)*qlin(l1) &
                      )
                 bnl=0.5d0*gaunt*( &
                      (l1*(l1+1d0)-l2*(l2+1d0)+l3*(l3+1d0))*clp(l1)*qnl(l3) &
                      +(l3*(l3+1d0)-l1*(l1+1d0)+l2*(l2+1d0))*clp(l3)*qnl(l2) &
                      +(l2*(l2+1d0)-l3*(l3+1d0)+l1*(l1+1d0))*clp(l2)*qnl(l1) &
                      +(l1*(l1+1d0)-l3*(l3+1d0)+l2*(l2+1d0))*clp(l1)*qnl(l2) &
                      +(l2*(l2+1d0)-l1*(l1+1d0)+l3*(l3+1d0))*clp(l2)*qnl(l3) &
                      +(l3*(l3+1d0)-l2*(l2+1d0)+l1*(l1+1d0))*clp(l3)*qnl(l1) &
                      )
                 num1a=num1a+blocal*blin/cl(l1)/cl(l2)/cl(l3)/delta(l1,l2,l3)
                 num2a=num2a+blocal*bnl/cl(l1)/cl(l2)/cl(l3)/delta(l1,l2,l3)
                 ! More accurate expression [Lewis, Challinor & Hanson (2011)]
                 ! clp -> cl, i.e., unlensed Cl -> lensed Cl
                 ! This is valid in the squeezed limit where l for cl(l) is small
                 blin=0.5d0*gaunt*( &
                      (l1*(l1+1d0)-l2*(l2+1d0)+l3*(l3+1d0))*cl(l1)*qlin(l3) &
                      +(l3*(l3+1d0)-l1*(l1+1d0)+l2*(l2+1d0))*cl(l3)*qlin(l2) &
                      +(l2*(l2+1d0)-l3*(l3+1d0)+l1*(l1+1d0))*cl(l2)*qlin(l1) &
                      +(l1*(l1+1d0)-l3*(l3+1d0)+l2*(l2+1d0))*cl(l1)*qlin(l2) &
                      +(l2*(l2+1d0)-l1*(l1+1d0)+l3*(l3+1d0))*cl(l2)*qlin(l3) &
                      +(l3*(l3+1d0)-l2*(l2+1d0)+l1*(l1+1d0))*cl(l3)*qlin(l1) &
                      )
                 bnl=0.5d0*gaunt*( &
                      (l1*(l1+1d0)-l2*(l2+1d0)+l3*(l3+1d0))*cl(l1)*qnl(l3) &
                      +(l3*(l3+1d0)-l1*(l1+1d0)+l2*(l2+1d0))*cl(l3)*qnl(l2) &
                      +(l2*(l2+1d0)-l3*(l3+1d0)+l1*(l1+1d0))*cl(l2)*qnl(l1) &
                      +(l1*(l1+1d0)-l3*(l3+1d0)+l2*(l2+1d0))*cl(l1)*qnl(l2) &
                      +(l2*(l2+1d0)-l1*(l1+1d0)+l3*(l3+1d0))*cl(l2)*qnl(l3) &
                      +(l3*(l3+1d0)-l2*(l2+1d0)+l1*(l1+1d0))*cl(l3)*qnl(l1) &
                      )
                 num1b=num1b+blocal*blin/cl(l1)/cl(l2)/cl(l3)/delta(l1,l2,l3)
                 num2b=num2b+blocal*bnl/cl(l1)/cl(l2)/cl(l3)/delta(l1,l2,l3)
              endif
           enddo
        endif
     enddo
     write(3,'(1I5,2E18.8)')l3,num1a/denom,num2a/denom
     write(4,'(1I5,2E18.8)')l3,num1b/denom,num2b/denom
  enddo
  close(3)
  close(4)
  print*,'fNL(contamination,leading)=',num1a/denom,num2a/denom
  print*,'fNL(contamination,accurate)=',num1b/denom,num2b/denom
  deallocate(qlin,qnl,cl,clp,wigner3j,r,dr,alpha,beta)
END PROGRAM compute_fNLcontamination_lensingisw
!---------------------------------------------------------
double precision function delta(l1,l2,l3)
! delta=6 for l1=l2=l3; delta=2 for l1=l2, l2=l3, or l3=l1; delta=1 otherwise
  implicit none
  integer, intent(IN) :: l1,l2,l3
  if((l1.eq.l2).and.(l2.eq.l3))then 
     delta=6d0
  elseif((l1.eq.l2).or.(l2.eq.l3).or.(l3.eq.l1))then 
     delta=2d0
  else
     delta=1d0
  endif
  return
END FUNCTION delta
